[general]
{% toml_field "apiEndpoint" endpoint_url %}
{% toml_field "apiEndpointText" config["api"]["text"] %}
{% toml_field "defaultSessionEnvironment" config["ui"]["default_environment"] %}
{% toml_field "siteDescription" config["ui"]["brand"] %}
connectionMode = "SESSION"

[wsproxy]
{% toml_field "proxyURL" config["service"]["wsproxy"]["url"] %}
# proxyBaseURL =
# proxyListenIP =
